import { NotUndefined } from 'type-fns';
import { SimpleCache } from '../../domain/SimpleCache';
import { WithSimpleCachingCacheOption } from '../options/getCacheFromCacheOption';
import { KeySerializationMethod } from '../serde/defaults';
/**
 * options to configure caching for use with-simple-caching
 */
export interface WithSimpleCachingAsyncOptions<
/**
 * the logic we are caching the responses for
 */
L extends (...args: any) => any, 
/**
 * the type of cache being used
 */
C extends SimpleCache<any>> {
    cache: WithSimpleCachingCacheOption<Parameters<L>, C>;
    serialize?: {
        key?: KeySerializationMethod<Parameters<L>>;
        value?: (output: Awaited<ReturnType<L>>) => NotUndefined<Awaited<ReturnType<C['get']>>>;
    };
    deserialize?: {
        value?: (cached: NotUndefined<Awaited<ReturnType<C['get']>>>) => Awaited<ReturnType<L>>;
    };
    secondsUntilExpiration?: number;
}
/**
 * a wrapper which adds asynchronous caching to asynchronous logic
 *
 * note
 * - utilizes an additional in-memory synchronous cache under the hood to prevent duplicate requests (otherwise, while async cache is resolving, a duplicate parallel request may have be made)
 * - can be given a synchronous cache, since what you can do on an asynchronous cache you can do on a synchronous cache, but not the other way around
 */
export declare const withSimpleCachingAsync: <L extends (...args: any[]) => Promise<any>, C extends SimpleCache<any>>(logic: L, { cache: cacheOption, serialize: { key: serializeKey, value: serializeValue, }, deserialize: { value: deserializeValue }, secondsUntilExpiration, }: WithSimpleCachingAsyncOptions<L, C>) => L;
